"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addRecentFile = addRecentFile;
exports["default"] = registerMenu;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _url = require("url");

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

var _electron = require("electron");

var _window = require("./window");

var _DataStorage = _interopRequireDefault(require("../DataStorage"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = o[Symbol.iterator](); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function getSavedRecentFile() {
  var recentFileName = "".concat(_DataStorage["default"].userDataDir, "/recent-opened-files.json");
  var content;

  try {
    content = _fs["default"].readFileSync(recentFileName, 'utf-8');
  } catch (e) {
    return [];
  }

  try {
    var arr = JSON.parse(content);
    if (!(arr instanceof Array)) return [];
    return arr;
  } catch (e) {
    return [];
  }
}

function saveRecentFile(file) {
  var recentFileName = "".concat(_DataStorage["default"].userDataDir, "/recent-opened-files.json");
  var arr = getSavedRecentFile();
  arr.push(file);

  _fs["default"].writeFileSync(recentFileName, JSON.stringify(arr), 'utf-8');
}

function addRecentFile(file) {
  var isSave = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

  var menu = _electron.Menu.getApplicationMenu();

  var itemRecentFiles = menu.getMenuItemById('recent-files');
  var item = new _electron.MenuItem({
    label: file.name,
    click: function click(menuItem, browserWindow) {
      // eslint-disable-next-line no-use-before-define
      openFile(browserWindow, file);
    }
  });
  itemRecentFiles.submenu.insert(0, item);

  _electron.Menu.setApplicationMenu(menu);

  if (isSave) saveRecentFile(file);
}

function recoverRecentFiles() {
  var arr = getSavedRecentFile();

  var _iterator = _createForOfIteratorHelper(arr),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var file = _step.value;
      addRecentFile(file, false);
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }
}

function onClickPreferences(browserWindow) {
  var window = browserWindow || (0, _window.getMainWindow)();
  var url = window.webContents.getURL();
  var urlInstance = new _url.URL(url);
  window.webContents.loadURL("".concat(urlInstance.origin, "/#/settings"));
}

function openFile(browserWindow, file) {
  if (!file) {
    var filePaths = _electron.dialog.showOpenDialog({
      title: 'Snapmaker Luban',
      filters: [{
        name: 'files',
        extensions: ['snap3dp', 'snaplzr', 'snapcnc', 'gcode', 'cnc', 'nc']
      }]
    });

    if (!filePaths) return;
    browserWindow.webContents.focus();
    file = {
      path: filePaths[0],
      name: _path["default"].basename(filePaths[0])
    };
    addRecentFile(file);
  }

  browserWindow.webContents.send('open-file', file);
}

function saveAsFile(browserWindow) {
  browserWindow.webContents.send('save-as-file');
}

function save(browserWindow) {
  browserWindow.webContents.send('save');
}

function getMenuTemplate(options) {
  var _options = _objectSpread({}, options),
      address = _options.address,
      port = _options.port;

  var template = [{
    label: 'File',
    submenu: [{
      label: 'Open File...',
      click: function click(menuItem, browserWindow, event) {
        openFile(browserWindow);
        console.log(event);
      }
    }, {
      id: 'recent-files',
      label: 'Recent Files',
      submenu: [{
        id: 'remove-recent',
        label: 'Clean All Recent Files',
        click: function click(menuItem) {
          var _iterator2 = _createForOfIteratorHelper(menuItem.menu.items),
              _step2;

          try {
            for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
              var item = _step2.value;
              if (item !== menuItem) item.visible = false;
            }
          } catch (err) {
            _iterator2.e(err);
          } finally {
            _iterator2.f();
          }

          var recentFileName = "".concat(_DataStorage["default"].userDataDir, "/recent-opened-files.json");

          _fs["default"].writeFileSync(recentFileName, JSON.stringify([]), 'utf-8');
        }
      }]
    }, {
      type: 'separator'
    }, {
      id: 'save',
      label: 'Save',
      accelerator: 'CommandOrControl+S',
      click: function click(menuItem, browserWindow) {
        save(browserWindow);
      }
    }, {
      id: 'save-as',
      label: 'Save As...',
      click: function click(menuItem, browserWindow) {
        saveAsFile(browserWindow);
      }
    }]
  }, {
    label: 'View',
    submenu: [{
      role: 'reload'
    }, {
      role: 'forcereload'
    }, {
      type: 'separator'
    }, {
      label: 'View In Browser',
      click: function click() {
        var url = "http://".concat(address, ":").concat(port);

        _electron.shell.openExternal(url);
      }
    }, {
      role: 'toggledevtools'
    }, {
      type: 'separator'
    }, {
      role: 'togglefullscreen'
    }]
  }, {
    role: 'window',
    submenu: [{
      role: 'minimize'
    }, {
      role: 'close'
    }]
  }];

  if (process.platform === 'darwin') {
    // About
    template.unshift({
      label: _electron.app.getName(),
      submenu: [{
        role: 'about'
      }, {
        type: 'separator'
      }, {
        label: 'Preferences...',
        accelerator: 'CommandOrControl+,',
        click: onClickPreferences
      }, {
        type: 'separator'
      }, {
        role: 'services',
        submenu: []
      }, {
        type: 'separator'
      }, {
        role: 'hide',
        label: 'Hide'
      }, {
        role: 'hideothers'
      }, {
        role: 'unhide'
      }, {
        type: 'separator'
      }, {
        role: 'quit',
        label: 'Quit'
      }]
    });
  }

  return template;
}

function registerMenu(options) {
  var template = getMenuTemplate(options);

  var menu = _electron.Menu.buildFromTemplate(template);

  _electron.Menu.setApplicationMenu(menu);

  recoverRecentFiles(); // Bind Menu defined shortcuts

  _electron.globalShortcut.register('CommandOrControl+,', onClickPreferences);
}